@@ -12,8 +12,9 @@ use vars qw(@ISA @EXPORT %dh);
 @ISA=qw(Exporter);
 @EXPORT=qw(&init &doit &complex_doit &verbose_print &error &warning &tmpdir
 	    &pkgfile &pkgext &pkgfilename &isnative &autoscript &filearray
-	    &filedoublearray &GetPackages &basename &dirname &xargs %dh
-	    &compat &addsubstvar &delsubstvar &excludefile);
+	    &filedoublearray &getpackages &basename &dirname &xargs %dh
+	    &compat &addsubstvar &delsubstvar &excludefile &package_arch
+	    &is_udeb &udeb_filename &debhelper_script_subst);
 
 my $max_compat=4;
 
@@ -72,7 +73,7 @@ sub init {
 		$dh{NO_ACT}=1;
 	}
 
-	my @allpackages=GetPackages();
+	my @allpackages=getpackages();
 	# Get the name of the main binary package (first one listed in
 	# debian/control). Only if the main package was not set on the
 	# command line.
@@ -99,6 +100,12 @@ sub init {
 	# This package gets special treatement: files and directories specified on
 	# the command line may affect it.
 	$dh{FIRSTPACKAGE}=${$dh{DOPACKAGES}}[0];
+
+	# If no error handling function was specified, just propigate
+	# errors out.
+	if (! exists $dh{ERROR_HANDLER} || ! defined $dh{ERROR_HANDLER}) {
+		$dh{ERROR_HANDLER}='exit $?';
+	}
 }
 
 # Pass it an array containing the arguments of a shell command like would
@@ -248,8 +255,14 @@ sub compat {
 	elsif (-e 'debian/compat') {
 		# Try the file..
 		open (COMPAT_IN, "debian/compat") || error "debian/compat: $!";
-		$c=<COMPAT_IN>;
-		chomp $c;
+		my $l=<COMPAT_IN>;
+		if (! defined $l || ! length $l) {
+			warning("debian/compat is empty, assuming level $c");
+		}
+		else {
+			chomp $l;
+			$c=$l
+		}
 	}
 
 	if ($c > $max_compat) {
@@ -533,8 +546,14 @@ sub excludefile {
 # Must pass "arch" or "indep" or "same" to specify arch-dependant or
 # -independant or same arch packages. If nothing is specified, returns all
 # packages.
-sub GetPackages {
+# As a side effect, populates %package_arches and %package_types with the
+# types of all packages (not only those returned).
+my (%package_types, %package_arches);
+sub getpackages {
 	my $type=shift;
+
+	%package_types=();
+	%package_arches=();
 	
 	$type="" if ! defined $type;
 	
@@ -546,6 +565,7 @@ sub GetPackages {
 
 	my $package="";
 	my $arch="";
+	my $package_type;
 	my @list=();
 	my %seen;
 	open (CONTROL, 'debian/control') ||
@@ -562,12 +582,20 @@ sub GetPackages {
 			else {
 				error("debian/control has a duplicate entry for $package");
 			}
+			$package_type="deb";
 		}
 		if (/^Architecture:\s*(.*)/) {
 			$arch=$1;
 		}
+		if (/^X[BC]*-Package-Type:\s*(.*)/) {
+			$package_type=$1;
+		}
 		
 		if (!$_ or eof) { # end of stanza.
+			if ($package) {
+				$package_types{$package}=$package_type;
+				$package_arches{$package}=$arch;
+			}
 			if ($package &&
 			    (($type eq 'indep' && $arch eq 'all') ||
 			     ($type eq 'arch' && $arch ne 'all') ||
@@ -584,4 +612,59 @@ sub GetPackages {
 	return @list;
 }
 
+# Returns the arch a package will build for.
+sub package_arch {
+	my $package=shift;
+	
+	return $package_arches{$package} eq 'all' ? "all" : buildarch();
+}
+
+# Return true if a given package is really a udeb.
+sub is_udeb {
+	my $package=shift;
+	
+	return $package_types{$package} eq 'udeb';
+}
+
+# Generates the filename that is used for a udeb package.
+sub udeb_filename {
+	my $package=shift;
+	
+	my $filearch=package_arch($package);
+	isnative($package); # side effect
+	my $version=$dh{VERSION};
+	$version=~s/^[0-9]+://; # strip any epoch
+	return "${package}_${version}_$filearch.udeb";
+}
+
+# Handles #DEBHELPER# substitution in a script; also can generate a new
+# script from scratch if none exists but there is a .debhelper file for it.
+sub debhelper_script_subst {
+	my $package=shift;
+	my $script=shift;
+	
+	my $tmp=tmpdir($package);
+	my $ext=pkgext($package);
+	my $file=pkgfile($package,$script);
+
+	if ($file ne '') {
+		if (-f "debian/$ext$script.debhelper") {
+			# Add this into the script, where it has #DEBHELPER#
+			complex_doit("perl -pe 's~#DEBHELPER#~qx{cat debian/$ext$script.debhelper}~eg' < $file > $tmp/DEBIAN/$script");
+		}
+		else {
+			# Just get rid of any #DEBHELPER# in the script.
+			complex_doit("sed s/#DEBHELPER#// < $file > $tmp/DEBIAN/$script");
+		}
+		doit("chown","0:0","$tmp/DEBIAN/$script");
+		doit("chmod",755,"$tmp/DEBIAN/$script");
+	}
+	elsif ( -f "debian/$ext$script.debhelper" ) {
+		complex_doit("printf '#!/bin/sh\nset -e\n' > $tmp/DEBIAN/$script");
+		complex_doit("cat debian/$ext$script.debhelper >> $tmp/DEBIAN/$script");
+		doit("chown","0:0","$tmp/DEBIAN/$script");
+		doit("chmod",755,"$tmp/DEBIAN/$script");
+	}
+}
+
 1
